"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import deleterunresponse as components_deleterunresponse
from typing import Optional


@dataclasses.dataclass
class DeleteRunRequest:
    run_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'run_id', 'style': 'simple', 'explode': False }})
    



@dataclasses.dataclass
class DeleteRunResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    delete_run_response: Optional[components_deleterunresponse.DeleteRunResponse] = dataclasses.field(default=None)
    r"""Successful response"""
    

